import logging
import timeit


def log_time(logger: logging.Logger):
    def inner(func):
        def wrapper(*args, **kwargs):
            start_time = timeit.default_timer()
            result = func(*args, **kwargs)
            elapsed = timeit.default_timer() - start_time
            logger.debug(f'function {func.__name__} took: {elapsed} seconds')
            return result

        return wrapper

    return inner



from typing import Callable

import pytest
from assertpy import assert_that

from common.polling import condition_polling


# noinspection PyTypeChecker
@pytest.mark.infra
@pytest.mark.parametrize('polling_func,expected_exception',
                         [(condition_polling.poll, TimeoutError),
                          (condition_polling.assert_poll, AssertionError)])
def test_polling_ignores_exception(polling_func: Callable, expected_exception):
    with pytest.raises(Exception) as e:
        polling_func(func=lambda: exec('raise(Exception("Test failed))'), ignore_exceptions=Exception,
                     timeout_in_sec=0.002, sleep_time_sec=0.001)
    assert_that(e.value).is_instance_of(expected_exception)


@pytest.mark.infra
@pytest.mark.parametrize('polling_func', [condition_polling.poll, condition_polling.assert_poll])
def test_polling_does_not_ignore_errors_by_default(polling_func):
    with pytest.raises(Exception) as e:
        polling_func(func=lambda: exec('raise TabError("This should be thrown")'),
                     timeout_in_sec=0.002, sleep_time_sec=0.001)
    assert_that(e.value).is_instance_of(TabError)


# noinspection PyTypeChecker
@pytest.mark.infra
@pytest.mark.parametrize('polling_func', [condition_polling.poll, condition_polling.assert_poll])
def test_polling_ignores_only_exceptions_of_given_types(polling_func):
    with pytest.raises(Exception) as e:
        polling_func(func=lambda: exec('raise TabError("This should be thrown")'),
                     ignore_exceptions=RecursionError,
                     timeout_in_sec=0.002, sleep_time_sec=0.001)
    assert_that(e.value).is_instance_of(TabError)


@pytest.mark.infra
@pytest.mark.parametrize('expected_value', [1, 'dummy', 0, None])
def test_polling_with_value_returns_actual_value(expected_value):
    value = condition_polling.poll_with_value(done_condition=lambda x: True, func=lambda x: x, args=(expected_value,))
    assert_that(value).is_equal_to(expected_value)


@pytest.mark.infra
@pytest.mark.parametrize('polling_func,expected_exception',
                         [(condition_polling.poll, TimeoutError),
                          (condition_polling.assert_poll, AssertionError)])
@pytest.mark.parametrize('exception_type', ['RecursionError', 'IOError'])
def test_polling_ignores_multiple_exceptions(polling_func: Callable, exception_type, expected_exception):
    with pytest.raises(Exception) as e:
        polling_func(func=lambda: exec(f'raise {exception_type}("This should be thrown")'),
                     ignore_exceptions=(RecursionError, IOError),
                     timeout_in_sec=0.002, sleep_time_sec=0.001)
    assert_that(e.value).is_instance_of(expected_exception)


@pytest.mark.infra
def test_assert_poll_with_value_raises_assertion_error():
    with pytest.raises(AssertionError) as e:
        condition_polling.assert_poll_with_value(done_condition=lambda x: False, func=lambda: "dummy",
                                                 sleep_time_in_sec=0.001, timeout_in_sec=0.002,
                                                 timeout_msg='Test passed')
    assert_that(e.value.args).contains_only('Test passed')


@pytest.mark.infra
def test_assert_poll_raises_assertion_error():
    with pytest.raises(AssertionError) as e:
        condition_polling.assert_poll(func=lambda: False, timeout_in_sec=0.001, sleep_time_sec=0.0005,
                                      timeout_msg='Test passed')
        assert_that(e.value.args).contains_only('Test passed')



import random

import re
import typing
from random import shuffle
from typing import Optional


def is_none_or_empty(s: Optional[str]) -> bool:
    return not s


def is_empty(s: str) -> bool:
    if s is None:
        raise ValueError('String is None')
    return len(s) == 0


def is_not_empty(s: Optional[str]) -> bool:
    return not is_none_or_empty(s)


def from_hex(value: str) -> int:
    return int(value, 16)


def version_to_tuple(version_str: str, ignore_non_digits_or_dots: bool = False) -> typing.List[int]:
    if not isinstance(version_str, str):
        raise TypeError(f'version_str should be a string and not {type(version_str)}')
    if is_empty(version_str):
        return []
    if ignore_non_digits_or_dots:
        version_str = re.sub(r'([^\d.]+)', '', version_str)
    if not re.match('^[0-9][0-9.]*$', version_str):
        raise ValueError(f'version string must be dot separated numbers only. Instead got {version_str}')
    return [int(x) for x in version_str.split('.')]


def remove_non_digit_characters(s: str):
    if s is None:
        return None
    return re.sub(r'([^\d]+)', '', s)


def find_all(substring, full_string):
    return [m.start() for m in re.finditer(substring, full_string)]


def shuffle_flags_for_commands(command):
    parts = command.split('/')
    new_command = parts.pop(0)
    shuffle(parts)
    for item in parts:
        new_command += f' /{item}'
    return new_command


def random_choice_string(lst: [str]):
    if lst is None or len(lst) is 0:
        return ''
    return random.choice(lst)



import os
import shutil
from urllib.parse import urlparse

import requests

"""
module for general http utils
"""


def validate_url(url):
    result = urlparse(url)
    if not all([result.scheme, result.netloc, result.path]):
        raise Exception(f'{url} is not a valid url')


def download_file(url, dest, is_https=True, delete_file_if_exists=True):
    """
    download file to local
    :param url:
    :param dest:
    :param is_https:
    :param delete_file_if_exists:
    :return:
    """
    validate_url(url)

    local_filename = os.path.join(dest, url.split('/')[-1])
    if delete_file_if_exists and os.path.isfile(local_filename):
        os.remove(local_filename)
    if is_https:
        r = requests.get(url, stream=True, verify=False)
    else:
        r = requests.get(url, stream=True)
    with open(local_filename, 'wb') as f:
        shutil.copyfileobj(r.raw, f)
    return local_filename



import time
import typing
from datetime import datetime

from common.logger import log_wrapper
from common.logger.disable_decorator import disable_logger

logger = log_wrapper.logger()


def poll(func: typing.Callable[..., bool], args: tuple = (), sleep_time_sec: float = 0.5, timeout_in_sec: float = 60,
         timeout_msg: str = 'timeout', ignore_exceptions: typing.Tuple = (), disable_log=False) -> None:
    if not is_condition_true(func=func, args=args, sleep_time_in_sec=sleep_time_sec, timeout_in_sec=timeout_in_sec,
                             ignore_exceptions=ignore_exceptions, disable_log=disable_log):
        raise TimeoutError(timeout_msg)


def is_condition_true(func: typing.Callable[..., bool], args: tuple = (), sleep_time_in_sec: float = 0.5,
                      timeout_in_sec: float = 60,
                      ignore_exceptions: typing.Tuple = (), disable_log=False) -> bool:
    is_successful, polling_result = _inner_poll_with_value(done_condition=lambda x: x, func=func, args=args,
                                                           sleep_time_in_sec=sleep_time_in_sec,
                                                           timeout_in_sec=timeout_in_sec,
                                                           ignore_exceptions=ignore_exceptions, disable_log=disable_log)
    return is_successful


def poll_with_value(done_condition: typing.Callable[..., bool], func: typing.Callable, args: tuple = (),
                    sleep_time_in_sec: float = 0.5,
                    timeout_in_sec: float = 60,
                    ignore_exceptions: typing.Tuple = (), timeout_msg: str = 'timeout') -> object:
    is_successful, polling_result = _inner_poll_with_value(done_condition=done_condition, func=func, args=args,
                                                           sleep_time_in_sec=sleep_time_in_sec,
                                                           timeout_in_sec=timeout_in_sec,
                                                           ignore_exceptions=ignore_exceptions)
    if not is_successful:
        raise TimeoutError(timeout_msg)
    return polling_result


def _inner_poll_with_value(done_condition: typing.Callable[..., bool], func: typing.Callable, args: tuple = (),
                           sleep_time_in_sec: float = 0.5,
                           timeout_in_sec: float = 60,
                           ignore_exceptions: typing.Tuple = (), disable_log: bool = False) -> typing.Tuple[
    bool, typing.Optional[object]]:
    logger.debug(f'running {func.__name__} going to wait for condition for {timeout_in_sec} seconds')

    @disable_logger(logger=logger, should_disable=disable_log)
    def __inner_poll():
        start_time = time.time()
        while time.time() - start_time < timeout_in_sec:
            try:
                logger.debug(f'checking condition: {func.__name__}')
                function_value = func(*args)
                if done_condition(function_value):
                    return True, function_value
            except Exception as e:
                if isinstance(e, ignore_exceptions):
                    logger.debug(f'Got exception while polling on condition, exception is <{str(e)}>')
                else:
                    raise e
            time.sleep(sleep_time_in_sec)

        return False, None

    return __inner_poll()


def assert_poll(func: typing.Callable[..., bool], args: tuple = (), sleep_time_sec: float = 0.5,
                timeout_in_sec: float = 60, timeout_msg: str = 'timeout', ignore_exceptions: typing.Tuple = ()) -> None:
    if not is_condition_true(func=func, args=args, sleep_time_in_sec=sleep_time_sec, timeout_in_sec=timeout_in_sec,
                             ignore_exceptions=ignore_exceptions):
        raise AssertionError(timeout_msg)


def assert_poll_with_value(done_condition: typing.Callable[..., bool], func: typing.Callable, args: tuple = (),
                           sleep_time_in_sec: float = 0.5,
                           timeout_in_sec: float = 60,
                           ignore_exceptions: typing.Tuple = (), timeout_msg: str = 'timeout') -> object:
    is_successful, polling_result = _inner_poll_with_value(done_condition=done_condition, func=func, args=args,
                                                           sleep_time_in_sec=sleep_time_in_sec,
                                                           timeout_in_sec=timeout_in_sec,
                                                           ignore_exceptions=ignore_exceptions)
    if not is_successful:
        raise AssertionError(timeout_msg)
    return polling_result


def change_timeout_by_time_of_day(timeout):
    now = datetime.utcnow().hour
    if int(now) > 17 or int(now) < 4:
        return timeout * 2
    return timeout



import logging


def disable_logger(logger: logging.Logger, should_disable=True):
    def inner(func):
        def wrapper(*args, **kwargs):
            if should_disable:
                logger.disabled = True
                result = func(*args, **kwargs)
                logger.disabled = False
                return result
            else:
                return func(*args, **kwargs)

        return wrapper

    return inner



import logging
import logging.config
from pathlib import Path

import yaml

config = None


def read_config_from_yaml():
    global config
    if config is None:
        path = str(Path(__file__).parent.parent) + '/logger/logging.yml'
        config = yaml.safe_load(open(path))
    return config


def logger(context='app') -> logging.Logger:
    logging.config.dictConfig(read_config_from_yaml())
    return logging.getLogger(context)



import pytest
from assertpy import assert_that

from common.utils import string_utils


@pytest.mark.infra
@pytest.mark.parametrize('version,expected_tuple', [('2.3.1', [2, 3, 1]),
                                                    ('1', [1, ]),
                                                    ])
def test_version_to_tuple(version: str, expected_tuple):
    assert_that(string_utils.version_to_tuple(version)).is_equal_to(expected_tuple)


@pytest.mark.infra
@pytest.mark.parametrize('version', [1, 1.1, {'1'}, {'1': 1}, ['list_type', ], ('1,1',)],
                         ids=['int', 'float', 'set', 'dict', 'list', 'tuple'])
def test_version_to_tuple_raises_exception_on_invalid_type(version: str, ):
    with pytest.raises(TypeError):
        string_utils.version_to_tuple(version)


@pytest.mark.infra
@pytest.mark.parametrize('version', ['a', '.1.1.1'])
def test_version_to_tuple_raises_exception_on_invalid_format(version: str, ):
    with pytest.raises(ValueError):
        string_utils.version_to_tuple(version)


@pytest.mark.infra
def test_version_to_tuple_remove_letters_does_not_bypass_other_exceptions():
    with pytest.raises(ValueError):
        string_utils.version_to_tuple('.1.2a', ignore_non_digits_or_dots=True)


@pytest.mark.infra
def test_version_to_tuple_ignore_non_digits_or_dots_parameter():
    version_str = '2a.1b.3p.145abcdefg.-1$%!^\\*&()'
    expected_result = [2, 1, 3, 145, 1]
    result = string_utils.version_to_tuple(version_str, ignore_non_digits_or_dots=True)
    assert_that(result).is_equal_to(expected_result)


@pytest.mark.infra
@pytest.mark.parametrize(argnames=['input_str', 'expected'],
                         argvalues=[['a,1/2.3.bb%4.5', '12345'],
                                    [None, None],
                                    ['', '']])
def test_remove_non_digit_characters(input_str, expected):
    result = string_utils.remove_non_digit_characters(input_str)
    assert_that(result).is_equal_to(expected)



import typing

import allure


@allure.step('Going to compare between {origin} and {array_to_compare}')
def compare_between_two_arrays(origin: typing.List, array_to_compare: typing.List, threshold: float) -> (typing.List,
                                                                                                         typing.List):
    origin_miss = []
    array_to_compare_miss = []
    for i in range(len(array_to_compare)):
        if origin[i] - array_to_compare[i] > threshold:
            origin_miss.append(origin[i])
            array_to_compare_miss.append(array_to_compare[i])
    return origin_miss, array_to_compare_miss



import setuptools

setuptools.setup(
    name="infra-common",
    version="1.1.0.6",
    author="Deep Instinct",
    author_email="qa@deepinstinct.com",
    description="common libraries for automation framework",
    long_description_content_type="text/markdown",
    package_dir={'common.logger': 'logger', 'common.utils': 'utils', 'common.polling': 'polling'},
    packages=['common.logger', 'common.utils', 'common.polling'],
    package_data={'': ['*.yml']},
    include_package_data=True,
    install_requires=['pyYAML'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
